#include "pslib.h"

// default constructor
cPSLib::cPSLib()
{
	sprintf(m_FileName, ""); 
	m_Fp = NULL; 
}

// constructor
cPSLib::cPSLib(char *FileName)
{
	OpenFile(FileName); 
}

// accessor functions
char *cPSLib::GetFileName()
{
	return m_FileName;
}

FILE *cPSLib::GetFilePointer()
{
	return m_Fp; 
}

// utility functions
void cPSLib::OpenFile(char *FileName)
{
	sprintf(m_FileName, "%s", FileName); 
	m_Fp = fopen(FileName, "w"); 
	fprintf(m_Fp, "%%!\n"); 
	fprintf(m_Fp, "%%postscript generated by pslib\n\n"); 
}

void cPSLib::CloseFile()
{
	fprintf(m_Fp, "\nshowpage\n"); 
	fclose(m_Fp); 
}

// drawing functions
void cPSLib::DrawLine(float x1, float y1, float x2, float y2, float width,
					  float color)
{
	fprintf(m_Fp, "\nnewpath\n");
	fprintf(m_Fp, "%f setlinewidth\n", width); 
	fprintf(m_Fp, "%f setgray\n", color);
	fprintf(m_Fp, "%f %f moveto\n", x1, y1); 
	fprintf(m_Fp, "%f %f lineto\n", x2, y2); 
	fprintf(m_Fp, "stroke\n");
}

void cPSLib::DrawCircle(float cx, float cy, float radius, 
						int fill, float width, float color)
{
	fprintf(m_Fp, "\nnewpath\n");
	fprintf(m_Fp, "%f setlinewidth\n", width); 
	fprintf(m_Fp, "%f %f %f 0 360 arc\n", cx, cy, radius);
	fprintf(m_Fp, "%f setgray\n", color);
	if (fill) fprintf(m_Fp, "fill\n"); 
	else fprintf(m_Fp, "stroke\n");
}

void cPSLib::DrawText(float x, float y, char *Text, int Scale, float color)
{
	fprintf(m_Fp, "\nnewpath\n"); 
	fprintf(m_Fp, "%f %f moveto\n", x, y); 
	fprintf(m_Fp, "/Times-Roman findfont %d scalefont setfont\n", Scale);
	fprintf(m_Fp, "%f setgray\n", color);
	fprintf(m_Fp, "(%s) show\n", Text); 
}

void cPSLib::DrawPolygon(float *x, float *y, int npoints, int fill, 
						 float width, float color)
{
	fprintf(m_Fp, "\nnewpath\n"); 
	fprintf(m_Fp, "%f setlinewidth\n", width); 
	fprintf(m_Fp, "%f setgray\n", color);
	fprintf(m_Fp, "%f %f moveto\n", x[0], y[0]);
	for (int i = 1; i < npoints; i++) {
		fprintf(m_Fp, "%f %f lineto\n", x[i], y[i]); 
	}
	fprintf(m_Fp, "closepath\n"); 
	if (fill) fprintf(m_Fp, "fill\n"); 
	else fprintf(m_Fp, "stroke\n");
}

// Zooming functions
void cPSLib::FitA4(float minx, float miny, float maxx, float maxy)
{
	float magx = maxx - minx; 
	float magy = maxy - miny;
	float scalex = (A4WIDTH - 20.0f) / magx;
	float scaley = (A4HEIGHT - 20.0f) / magy;
	float ScaleFactor;

	if (scalex < scaley) ScaleFactor = scalex;
	else ScaleFactor = scaley;

	fprintf(m_Fp, "\n%%Fitting regions to A4 paper\n"); 
	fprintf(m_Fp, "%f %f scale\n\n", ScaleFactor, ScaleFactor); 
	fprintf(m_Fp, "%f %f translate\n", -minx, -miny);
	fprintf(m_Fp, "10 10 translate\n"); 
}
